# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,spec)
S3method(as.data.frame,tapers)
S3method(as.list,spec)
S3method(as.matrix,spec)
S3method(constrain_tapers,default)
S3method(constrain_tapers,tapers)
S3method(ctap_loess,default)
S3method(ctap_loess,tapers)
S3method(ctap_simple,default)
S3method(ctap_simple,tapers)
S3method(lines,spec)
S3method(lines,tapers)
S3method(minspan,default)
S3method(minspan,tapers)
S3method(na_locf,default)
S3method(na_locf,matrix)
S3method(normalize,amt)
S3method(normalize,list)
S3method(normalize,spec)
S3method(parabolic_weights,default)
S3method(parabolic_weights,tapers)
S3method(pgram_compare,amt)
S3method(pilot_spec,default)
S3method(pilot_spec,matrix)
S3method(pilot_spec,ts)
S3method(plot,tapers)
S3method(points,tapers)
S3method(prewhiten,default)
S3method(prewhiten,ts)
S3method(print,summary.tapers)
S3method(print,tapers)
S3method(psdcore,default)
S3method(psdcore,matrix)
S3method(psdcore,ts)
S3method(pspectrum,default)
S3method(pspectrum,matrix)
S3method(pspectrum,spec)
S3method(pspectrum,ts)
S3method(riedsid,default)
S3method(riedsid,spec)
S3method(riedsid2,default)
S3method(riedsid2,spec)
S3method(spec_confint,default)
S3method(spec_confint,spec)
S3method(spec_confint,tapers)
S3method(spectral_properties,default)
S3method(spectral_properties,spec)
S3method(spectral_properties,tapers)
S3method(splineGrad,default)
S3method(summary,tapers)
S3method(update_adapt_history,default)
S3method(update_adapt_history,spec)
S3method(varddiff,default)
S3method(varddiff,spec)
export(.spec_confint)
export(adapt_message)
export(as.tapers)
export(coherence)
export(colvec)
export(constrain_tapers)
export(create_poly)
export(ctap_loess)
export(ctap_simple)
export(dB)
export(data.frame.tapers)
export(det_vector)
export(get_adapt_history)
export(get_psd_env_name)
export(get_psd_env_pointer)
export(is.amt)
export(is.spec)
export(is.tapers)
export(last_psd)
export(minspan)
export(mod)
export(modulo_floor)
export(na_locf)
export(na_mat)
export(new_adapt_history)
export(normalize)
export(ones)
export(parabolic_weights)
export(parabolic_weights_field)
export(parabolic_weights_rcpp)
export(pgram_compare)
export(phase)
export(pilot_spec)
export(prewhiten)
export(psd_envAssign)
export(psd_envAssignGet)
export(psd_envClear)
export(psd_envGet)
export(psd_envList)
export(psd_envRefresh)
export(psd_envStatus)
export(psdcore)
export(pspectrum)
export(pspectrum_basic)
export(rcpp_ctap_simple)
export(resample_fft_rcpp)
export(resample_mvfft)
export(riedsid)
export(riedsid2)
export(riedsid_rcpp)
export(rowvec)
export(spec_confint)
export(spec_details)
export(spectral_properties)
export(splineGrad)
export(tapers)
export(update_adapt_history)
export(varddiff)
export(vardiff)
export(vector_reshape)
export(zeros)
import(RColorBrewer)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,abline)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,title)
importFrom(stats,acf)
importFrom(stats,as.ts)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,pchisq)
importFrom(stats,qchisq)
importFrom(stats,residuals)
importFrom(stats,spec.pgram)
importFrom(stats,start)
importFrom(stats,ts)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(psd)
